/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 24/11/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistry.servlet;

import eu.europa.ec.re3gistry.constants.Constants;
import eu.europa.ec.re3gistry.utils.Application;
import eu.europa.ec.re3gistry.utils.locale.LocalizationMgr;
import java.io.IOException;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;
import eu.cec.digit.ecas.client.jaas.DetailedUser;
import eu.europa.ec.re3gistrycommon.managers.UserMgr;
import org.apache.shiro.SecurityUtils;

/**
 *
 * @author francdb
 */
public class Login extends HttpServlet {

    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code>
     * methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType(Constants.CONTENT_TYPE);
        request.setAttribute(Constants.ERROR_KEY, null);

        //Inizializing the logger
        Logger log = Logger.getLogger(Constants.BASEAPP_NAME);
        request.setAttribute(Constants.LOG_KEY, log);

        try {

            String loginParam = request.getParameter(Constants.LOGIN_PARAMETER_KEY);

            /**
             * Initializing elements *
             */
            //Initializes the properties related to the main module
            Properties props = Application.getProperties();
            request.setAttribute(Constants.PROPS_KEY, props);

            //Initializing baseurl
            String baseurl = Application.getBasepath(request);
            request.setAttribute(Constants.BASEURL_KEY, baseurl);

            //Initializing resource bundle (localization)
            ResourceBundle localization = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.BASEAPP_NAME + Constants.LOCALIZATION_SUFFIX, LocalizationMgr.getCurrentLocale(request));
            request.setAttribute(Constants.LOCALIZATION_KEY, localization);

            //Setting the current locale
            Locale currentLocale = LocalizationMgr.getCurrentLocale(request);
            request.setAttribute(Constants.CURRENTLOCALE_KEY, currentLocale);

            //Setting the available language site
            List availableLanguages = LocalizationMgr.getAvailableLanguages();
            request.setAttribute(Constants.AVAILABLELANGUAGES_KEY, availableLanguages);

            //Get the current Login Type from the configuration
            String loginType = props.getProperty(Constants.LOGIN_TYPE_PROPERTY_KEY, Constants.LOGIN_TYPE_SHIRO);

            //Handring ECAS login type
            if (loginType.equals(Constants.LOGIN_TYPE_ECAS)) {

                if (loginParam != null) {
                    UserMgr userMgr = new UserMgr(null);
                    response.sendRedirect(userMgr.getLogoutLink(request));
                } else {
                    //// Checking login status ////
                    DetailedUser detailedUser = (DetailedUser) request.getUserPrincipal();
                    if (detailedUser == null) {
                        request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                    } else {
                        request.getRequestDispatcher(Constants.MAIN_PAGE_FORWARD).forward(request, response);
                    }
                }

            //Handling SHIRO login type   
            } else if (loginType.equals(Constants.LOGIN_TYPE_SHIRO)) {
                if (loginParam != null) {
                    SecurityUtils.getSubject().logout();
                    if(request.getSession()!=null){
                        request.getSession().invalidate();
                    }
                    request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                } else {

                    Object o = SecurityUtils.getSubject().getPrincipal();
                    if (o == null) {
                        request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                    } else {
                        request.getRequestDispatcher(Constants.MAIN_PAGE_FORWARD).forward(request, response);
                    }
                }
            } else {
                throw new Exception("Configuration error: login type not properly configured.");
            }

        } catch (Exception e) {
            log.error("Error during the page initialization. Message: " + e.getMessage());
            request.getRequestDispatcher(Constants.ERROR_GENERIC).forward(request, response);
        } finally {
        }
    }

    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }
}
