/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 30/03/2015
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller;

import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrystaticizer.StaticizeFiles;
import eu.europa.ec.re3gistrystaticizer.util.Constants;
import eu.europa.ec.re3gistrystaticizer.marshaller.custom.RegisterCustomFiles;
import eu.europa.ec.re3gistrystaticizer.marshaller.custom.RegistryCustomFiles;
import eu.europa.ec.re3gistrystaticizer.marshaller.custom.ItemsCustomFiles;
import eu.europa.ec.re3gistrystaticizer.marshaller.ror.RegisterROR;
import eu.europa.ec.re3gistrystaticizer.marshaller.ror.RegistryROR;
import java.io.File;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;

public class CustomFormatsFilesCreation {

    private String staticizerStandardFolder;
    private String staticizerCutomFolder;
    private String folderStandardStaticizer;
    private String folderCustomStaticizer;
    private final EntityManager em;
    private final ResourceBundle localizationres;
    private final ReportMgr reportMgr;

    private String[] EXTENSIONS;
    private final StaticizeFiles staticizeFiles;

    private final String folderSeparator = System.getProperty("file.separator");
    private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);

    public CustomFormatsFilesCreation(StaticizeFiles staticizeFiles, LinkedHashMap<Item, List<Languagecode>> itemsTostaticize, LinkedHashMap<File, String> linkedHashMapVAR, LinkedHashMap<Itemclass, List<Languagecode>> registerToStaticize, EntityManager em) throws Exception {
        this.staticizeFiles = staticizeFiles;
        this.em = em;

        this.localizationres = staticizeFiles.getLocalizationres();
        this.reportMgr = staticizeFiles.getReportMgr();
    }

    public void initAndCreateStardardFile(LinkedHashMap<Item, List<Languagecode>> itemsTostaticize, LinkedHashMap<File, String> linkedHashMapVAR, LinkedHashMap<Itemclass, List<Languagecode>> registerToStaticize, EntityManager em1) throws Exception {
        initalizateStaticizerStandardAndCustomFolders();
        if (EXTENSIONS.length != 0 && !EXTENSIONS[0].equals("")) {
            createStandardFiles(itemsTostaticize, linkedHashMapVAR, registerToStaticize, em1);
        }
    }

    private void initalizateStaticizerStandardAndCustomFolders() throws Exception {
        /**
         * initialize the staticizer standard and custom folder
         */
        Properties properties = eu.europa.ec.re3gistrycommon.utils.Configuration.getInstance().getProperties(Constants.MODULE_NAME);

        staticizerStandardFolder = properties.getProperty(Constants.STATICIZER_STANDARD_FOLDER, "standard");
        folderStandardStaticizer = staticizeFiles.getStaticizerMainfolderName() + folderSeparator + staticizerStandardFolder;
        File stardardFolder = new File(folderStandardStaticizer);
        if (!stardardFolder.exists()) {
            boolean makeStardardFolder = stardardFolder.mkdirs();
            if (!makeStardardFolder) {

                String subject = localizationres.getString("export.mail.subject.error");
                String body = localizationres.getString("export.mail.body.staticizer.standard.folder").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                reportMgr.setMailSubject(subject);
                reportMgr.setMailBody(body);
                reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("The path 'staticizer.standard.folder' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.error("===================The path 'staticizer.standard.folder' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.====================");
                throw new Exception("The path 'staticizer.standard.folder' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.");
            }
        }

        staticizerCutomFolder = properties.getProperty(Constants.STATICIZER_CUSTOM_FOLDER, "custom");
        folderCustomStaticizer = staticizeFiles.getStaticizerMainfolderName() + folderSeparator + staticizerCutomFolder;
        File customfolder = new File(folderCustomStaticizer);
        if (!customfolder.exists()) {
            boolean makeCustomFolder = customfolder.mkdir();
            if (!makeCustomFolder) {

                String subject = localizationres.getString("export.mail.subject.error");
                String body = localizationres.getString("export.mail.body.staticizer.custom.folder").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                reportMgr.setMailSubject(subject);
                reportMgr.setMailBody(body);
                reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("The path 'staticizer.custom.folder' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.error("===================The path 'staticizer.custom.folder' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.====================");
                throw new Exception("The path 'staticizer.custom.folder' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.");
            }
        }

        String formatList = properties.getProperty(Constants.STATICIZER_FORMATS);
        EXTENSIONS = formatList.split(properties.getProperty(Constants.STATICIZER_FORMATS_SEPARATOR));
    }

    private ItemsCustomFiles createStandardFiles(LinkedHashMap<Item, List<Languagecode>> itemsTostaticize, LinkedHashMap<File, String> linkedHashMap, LinkedHashMap<Itemclass, List<Languagecode>> registerToStaticize, EntityManager em) throws Exception {
        /**
         * read/write the items
         */
        List<File> folderItemList = new ArrayList<File>();
        LocalizationMgr localizationMgr = new LocalizationMgr(em);
        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        Languagecode masterLanguage = languagecodeMgr.getMasterLanguagecode();

        ItemsCustomFiles staticizeLastImportedItems = new ItemsCustomFiles(staticizeFiles, em);
        for (Map.Entry<Item, List<Languagecode>> entry : itemsTostaticize.entrySet()) {
            Item item = entry.getKey();
            List<Languagecode> list = entry.getValue();
            for (Languagecode languagecode : list) {
                Localization itemLocalization;
                try {
                    itemLocalization = localizationMgr.findByItemAndLanguagecode(item, languagecode);
                    if (itemLocalization != null) {
                        folderItemList.addAll(staticizeLastImportedItems.createItemFiles(item, itemLocalization, languagecode));
                    } else {
                        itemLocalization = localizationMgr.findByItemAndLanguagecode(item, masterLanguage);
                        folderItemList.addAll(staticizeLastImportedItems.createItemFiles(item, itemLocalization, languagecode));
                    }
                } catch (Exception ex) {
                    itemLocalization = localizationMgr.findByItemAndLanguagecode(item, masterLanguage);
                    folderItemList.addAll(staticizeLastImportedItems.createItemFiles(item, itemLocalization, languagecode));
                }
            }
        }

        /**
         * create the register
         */
        for (Map.Entry<Itemclass, List<Languagecode>> entry : registerToStaticize.entrySet()) {
            Itemclass itemclass = entry.getKey();
            List<Languagecode> list = entry.getValue();
            for (Languagecode languagecode : list) {
                RegisterCustomFiles registerStaticizer = new RegisterCustomFiles(staticizeFiles, em);
                folderItemList.add(registerStaticizer.addRegisterFiles(itemclass.getRegister(), languagecode));
            }
        }

        /**
         * create the registry
         */
        RegistryCustomFiles registryStaticizer = new RegistryCustomFiles(staticizeFiles, em);
        folderItemList.addAll(registryStaticizer.addRegistryFiles());

        /**
         * ROR register descriptor
         */
        RegisterROR registerROR = new RegisterROR(staticizeFiles, em);
        folderItemList.addAll(registerROR.addRegisterFiles(registerToStaticize, itemsTostaticize));
               
        /**
         * ROR registry descriptor
         */
        RegistryROR registryROR = new RegistryROR(staticizeFiles, em);
        folderItemList.addAll(registryROR.addRegistryFiles());
        
        for (File file : folderItemList) {
            if (!linkedHashMap.containsKey(file)) {
                linkedHashMap.put(file, "");
            }
        }

        
        return staticizeLastImportedItems;
    }

    public String getFolderCustomStaticizer() {
        return folderCustomStaticizer;
    }

    public String getFolderStandardStaticizer() {
        return folderStandardStaticizer;
    }

    public String[] getEXTENSIONS() {
        return EXTENSIONS;
    }

    public StaticizeFiles getStaticizeFiles() {
        return staticizeFiles;
    }

}
