/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: --/--/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller.ror;

import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.managers.RegisterMgr;
import eu.europa.ec.re3gistrycommon.managers.RegistryMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrycommon.model.Registry;
import eu.europa.ec.re3gistrycommon.utils.StringUtils;
import eu.europa.ec.re3gistrystaticizer.StaticizeFiles;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegisterRORStandard;
import eu.europa.ec.re3gistrystaticizer.util.Constants;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegistryRORStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DefinitionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DescriptionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.LabelAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.list.RegisterRORStandardList;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.EntityManager;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.TransformerFactoryImpl;

public class RegistryROR {

    private final EntityManager em;

    private final String folderSeparator;
    private final String folderStandardStaticizer;

    private final String folderCustomStaticizer;
    private final StaticizeFiles staticizeFiles;

    private final List<Languagecode> allLanguagecode;
    private final Languagecode masterLanguage;

    private static org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);
    private final ReportMgr reportMgr;

    /**
     *
     * @param staticizeFiles
     * @param em
     * @throws java.lang.Exception
     */
    public RegistryROR(StaticizeFiles staticizeFiles, EntityManager em) throws Exception {
        this.staticizeFiles = staticizeFiles;
        this.folderSeparator = staticizeFiles.getFolderSeparator();
        this.folderStandardStaticizer = staticizeFiles.getCreateStandardFiles().getFolderStandardStaticizer();
        this.folderCustomStaticizer = staticizeFiles.getCreateStandardFiles().getFolderCustomStaticizer();

        this.reportMgr = staticizeFiles.getReportMgr();
        this.em = em;

        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        allLanguagecode = languagecodeMgr.findAll();

        masterLanguage = languagecodeMgr.getMasterLanguagecode();
    }

    public List<File> addRegistryFiles() throws Exception {
        List<File> list = new ArrayList<File>();
        if (staticizeFiles.getCreateStandardFiles().getEXTENSIONS().length != 0 && !staticizeFiles.getCreateStandardFiles().getEXTENSIONS()[0].equals("")) {
            try {
                RegistryMgr registryMgr = new RegistryMgr(em);
                List<Registry> registryList = registryMgr.findAll();

                RegisterMgr registerMgr = new RegisterMgr(em);
                ItemclassMgr itemclassMgr = new ItemclassMgr(em);
                ItemMgr itemMgr = new ItemMgr(em);

                for (Registry registry : registryList) {

                    List<Register> registerList = registerMgr.findRegisterByRegistry(registry);
                    ArrayList<RegisterRORStandard> registerRORStandards = new ArrayList<RegisterRORStandard>();
                    for (Register register : registerList) {

                        /**
                         * check the items of the register
                         */
                        Itemclass itemclassFirstLevel = itemclassMgr.getItemClassFirstLevelIfRegisterHierarchical(register);
                        if (itemclassFirstLevel != null) {
                            /**
                             * get all the items with itemclass
                             * itemclassFirstLevel
                             */
                            List<Item> itemsWithItemclassFirstLevel = itemMgr.findItemsByItemclass(itemclassFirstLevel);
                            for (Item item : itemsWithItemclassFirstLevel) {

                                if (!item.getUriname().contains("//")) {
                                    RegisterRORStandard registerRORStandard = new RegisterRORStandard();
                                    final String uri = item.getItemclass().getRegister().getBaseuri() + "/" + item.getItemclass().getRegister().getUriname() + "/" + item.getUriname();
                                    registerRORStandard.setUri(uri);
                                    registerRORStandard.setUri_descriptor(uri + "/" + item.getUriname() + ".ror");
                                    registerRORStandards.add(registerRORStandard);
                                }
                            }
                        } else {
                            RegisterRORStandard registerRORStandard = new RegisterRORStandard();
                            registerRORStandard.setUri(register.getBaseuri() + "/" + register.getUriname());
                            registerRORStandard.setUri_descriptor(register.getBaseuri() + "/" + register.getUriname() + "/" + register.getUriname() + ".ror");
                            registerRORStandards.add(registerRORStandard);
                        }

                    }
                    RegisterRORStandardList registerRORStandardList = new RegisterRORStandardList();
                    registerRORStandardList.setRegisterRORStandard(registerRORStandards);

                    RegistryRORStandard registryRORStandard = new RegistryRORStandard();
                    registryRORStandard.setUri(registry.getBaseuri() + "/" + registry.getUriname());
                    registryRORStandard.setUriname(registry.getUriname());
                    registryRORStandard.setRegisterRORStandardList(registerRORStandardList);

                    LocalizationMgr localizationMgr = new LocalizationMgr(em);
                    Localization registryRORLocalization = localizationMgr.findByRegistryAndLanguagecode(registry, masterLanguage);
                    String languageLocalization = registryRORLocalization.getLanguage().getIsocode();

                    LabelAttribute registryLabelAttribute = new LabelAttribute();
                    registryLabelAttribute.setLanguage(languageLocalization);
                    registryLabelAttribute.setValue(registryRORLocalization.getLabel());
                    registryRORStandard.setLabel(registryLabelAttribute);

                    StringUtils stringUtils = new StringUtils();

                    if (registryRORLocalization.getDefinition() != null) {
                        DefinitionAttribute registryDefinitionAttribute = new DefinitionAttribute();
                        registryDefinitionAttribute.setLanguage(languageLocalization);

                        String definition = stringUtils.replaceSpecialCharacters(registryRORLocalization.getDefinition());
                        registryDefinitionAttribute.setValue(definition);
                        registryRORStandard.setDefinition(registryDefinitionAttribute);
                    }

                    if (registryRORLocalization.getDescription() != null) {
                        DescriptionAttribute registryDescriptionAttribute = new DescriptionAttribute();
                        registryDescriptionAttribute.setLanguage(languageLocalization);

                        String description = stringUtils.replaceSpecialCharacters(registryRORLocalization.getDescription());
                        registryDescriptionAttribute.setValue(description);
                        registryRORStandard.setDescription(registryDescriptionAttribute);
                    }

                    list.add(createXMLRegistryRORFile(registryRORStandard, masterLanguage.getIsocode()));
                }
            } catch (Exception ex) {
                String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                staticizeFiles.getReportMgr().setMailSubject(subject);
                staticizeFiles.getReportMgr().setMailBody(body);
                staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("Error in writing Standard && Custom Item Files for the Registry - RegistryROR", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.info("=================================Error in writing Standard && Custom Item Files for the Registry - RegistryROR=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files  for the Registry. ");
            }
        }
        return list;
    }

    private File createXMLRegistryRORFile(RegistryRORStandard registryRORStandard, String isocode) throws Exception {
        String registryStandardFolderPath = folderStandardStaticizer;
        String registryCustomFolderPath = folderCustomStaticizer;
        File registryStandardFolder = new File(registryStandardFolderPath);
        File registryCustomFolder = new File(registryCustomFolderPath);

        if (!registryStandardFolder.exists()) {
            registryStandardFolder.mkdirs();
        }
        if (!registryCustomFolder.exists()) {
            registryCustomFolder.mkdirs();
        }

        String registry = registryRORStandard.getUriname();
        String registryFilePath = registryStandardFolder.getAbsolutePath() + folderSeparator + registry + /*".descriptor."  + isocode +*/ ".xml";
        File registryFile = new File(registryFilePath);

        try {
            /**
             * STANDARD XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(RegistryRORStandard.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshallerclassLanguage.marshal(registryRORStandard, registryFile);
            log.info("Create with success: " + registryFile);

            String ext = "ror";
            try {
                String registryXSL = staticizeFiles.getStaticizerXSLFolder() + "/" + ext + "/" + registry + "." + ext + ".xsl";
                String finalOutputPath = registryFile.getAbsolutePath().replace("standard", "custom").replace("xml", ext);

                File finalOutFile = new File(finalOutputPath);
                finalOutFile.createNewFile();
                File transformerFile = new File(registryXSL);

                TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
                Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                transformer.transform(new StreamSource(registryFile), new StreamResult(finalOutFile));
                log.info("Create with success: " + finalOutFile);
            } catch (Exception ex) {
                String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                staticizeFiles.getReportMgr().setMailSubject(subject);
                staticizeFiles.getReportMgr().setMailBody(body);
                staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files. ");
            }

        } catch (JAXBException ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom item files. ");
        }

        return registryCustomFolder;
    }
}
