/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 30/03/2015
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller.solr;

import eu.europa.ec.re3gistrystaticizer.marshaller.SOLRFilesCreation;
import eu.europa.ec.re3gistrycommon.managers.ItemcollectionMgr;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrystaticizer.util.Constants;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.collection.ContainerCollectionItemStandard;
import java.io.File;
import javax.persistence.EntityManager;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.TransformerFactoryImpl;
import org.apache.log4j.Logger;

public class ItemsSolrFiles {

    private final EntityManager em;

    private final SOLRFilesCreation createSOLRFiles;
    private final String folderSolrStaticizer;
    private final String folderSolrcustomStaticizer;

    private final String folderSeparator = System.getProperty("file.separator");
    private static Logger log = Logger.getLogger(Constants.MODULE_NAME);

    public ItemsSolrFiles(SOLRFilesCreation createSOLRFiles, EntityManager em) {
        this.createSOLRFiles = createSOLRFiles;

        this.folderSolrStaticizer = createSOLRFiles.getFolderSolrStaticizer();
        this.folderSolrcustomStaticizer = createSOLRFiles.getFolderSolrCustomStaticizer();

        this.em = em;
    }

    public File createItemStandardSOLRFile(Item item, ContainerCollectionItemStandard containerCollectionItemStandard) {
        Register register = item.getItemclass().getRegister();
        String registerUriname = register.getUriname();

        String registerSolrFolderPath = folderSolrStaticizer + folderSeparator + registerUriname;
        String registerSolrCustomFolderPath = folderSolrcustomStaticizer + folderSeparator + registerUriname;
        File registerSolrFolder = new File(registerSolrFolderPath);
        File registerSolrCustomFolder = new File(registerSolrCustomFolderPath);

        if (!registerSolrFolder.exists()) {
            registerSolrFolder.mkdirs();
        }
        if (!registerSolrCustomFolder.exists()) {
            registerSolrCustomFolder.mkdirs();
        }

        String itemVersionStandardFolderPath = null;
        ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(em);
        File fileSolr = null;
        File fileSolrCustom = null;

        try {
            Itemcollection itemcollection = itemcollectionMgr.findByItem(item).get(0);

            if (itemcollection != null) {
                Item collection = itemcollection.getCollection();
                if (item.getVersionnumber() == 0) {
                    fileSolr = new File(registerSolrFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname());
                    fileSolrCustom = new File(registerSolrCustomFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname());
                    fileSolr.mkdirs();
                    fileSolrCustom.mkdirs();
                    itemVersionStandardFolderPath = fileSolr.getAbsolutePath() + folderSeparator + item.getUriname() + ".xml";
                } else {
                    fileSolr = new File(registerSolrFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname() + folderSeparator + item.getVersionnumber());
                    fileSolrCustom = new File(registerSolrCustomFolderPath + folderSeparator + collection.getUriname() + folderSeparator + item.getUriname() + folderSeparator + item.getVersionnumber());
                    fileSolr.mkdirs();
                    fileSolrCustom.mkdirs();
                    itemVersionStandardFolderPath = fileSolr.getAbsolutePath() + folderSeparator + item.getUriname() + ".xml";
                }
            }
        } catch (Exception ex) {
            if (item.getVersionnumber() == 0) {
                fileSolr = new File(registerSolrFolderPath + folderSeparator + item.getUriname());
                fileSolrCustom = new File(registerSolrCustomFolderPath + folderSeparator + item.getUriname());
                fileSolr.mkdirs();
                fileSolrCustom.mkdirs();
                itemVersionStandardFolderPath = registerSolrFolderPath + folderSeparator + item.getUriname() + folderSeparator + item.getUriname() + ".xml";
            } else {
                fileSolr = new File(registerSolrFolderPath + folderSeparator + item.getUriname() + folderSeparator + item.getVersionnumber());
                fileSolrCustom = new File(registerSolrCustomFolderPath + folderSeparator + item.getUriname() + folderSeparator + item.getVersionnumber());
                fileSolr.mkdirs();
                fileSolrCustom.mkdirs();
                itemVersionStandardFolderPath = fileSolr.getAbsolutePath() + folderSeparator + item.getUriname() + ".xml";
            }
        }
        File itemVersionFile = new File(itemVersionStandardFolderPath);

        createSOLRFile(containerCollectionItemStandard, itemVersionFile);

        return fileSolrCustom;
    }

    private void createSOLRFile(ContainerCollectionItemStandard containerCollectionItemStandard, File itemSolrFile) {
        String itemFileUriname = containerCollectionItemStandard.getCollectionItemStandard().getItemStandard().get(0).getItemclass().getUriname();

        try {
            /**
             * STANDARD SOLR XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(ContainerCollectionItemStandard.class);
            Marshaller marshaller = jaxbContextLang.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshaller.marshal(containerCollectionItemStandard, itemSolrFile);
            log.info("Create with success: " + itemSolrFile);

            /**
             * CUSTOM SOLR
             */
            for (String ext : createSOLRFiles.getEXTENSIONSSOLR()) {
                try {
                    String itemFileXSL = createSOLRFiles.getStaticizeFiles().getStaticizerXSLFolder() + "/" + ext + "/" + itemFileUriname + "." + ext + ".xsl";
                    String finalOutputPath = itemSolrFile.getAbsolutePath().replace("solr", "solrcustom");

                    File finalOutFile = new File(finalOutputPath);
                    finalOutFile.createNewFile();
                    File transformerFile = new File(itemFileXSL);

                    TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
                    Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                    transformer.transform(new StreamSource(itemSolrFile), new StreamResult(finalOutFile));

                    log.info("Create with success: " + finalOutFile);
                } catch (Exception ex) {
                    log.error("Error in creation the transformation file: " + itemSolrFile + " == " + ex.getMessage());
                }
            }

        } catch (JAXBException ex) {
            log.error("Error in creation file: " + itemSolrFile + " -JAXBException- " + ex.getMessage());
        }
    }
}
